function [ X2, score ] = f_get_similarity_third_order( P1, P2, X )
%f_get_similarity_third_order: this function is used to generate similarity
%between two sets of points based on the high-order graph matching method.
%   Author: Cong Yang
%   Please cite the following paper: 
%   C. Yang, etc, Shape-based Object Matching Using Point Context, ICMR 2015, pages 519-522.

%   input:
%         P1: the first point set
%         P2: the second point set
%   output:
%         X2: similarity matrix between all point pairs
%         score: similarity between two sets of point


nP1 = size(P1,2);
nP2 = size(P2,2);

%number of used triangles (results can be bad if too low)
nT=nP1*50;
t1=floor(rand(3,nT)*nP1);
while 1
  probFound=false;
  for i=1:3
    ind=(t1(i,:)==t1(1+mod(i,3),:));
    if(nnz(ind)~=0)
      t1(i,ind)=floor(rand(1,nnz(ind))*nP1);
      probFound=true;
    end
  end
  if(~probFound)
    break;
  end
end

%generate features
t1=int32(t1);
[feat1,feat2] = mexComputeFeature(P1,P2,int32(t1),'simple');

%number of nearest neighbors used for each triangle (results can be bad if
%too low)
nNN=64;

%find the nearest neighbors
[inds, dists] = annquery(feat2, feat1, nNN, 'eps', 10);

%build the tensor
[i j k]=ind2sub([nP2,nP2,nP2],inds);
tmp=repmat(1:nT,nNN,1);
indH = t1(:,tmp(:))'*nP2 + [k(:)-1 j(:)-1 i(:)-1];
valH = exp(-dists(:)/mean(dists(:)));

%initiatialize X
%X=1/nP2*ones(nP2,nP1);

%power iteration
[X2, score]=tensorMatching(X,[],[],[],[],indH,valH);
score = roundn(score,0);
end

